﻿using System;
using System.Collections.Generic;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using MvvmDataBindingDemo;
using MvvmDataBindingDemo.DomainModel;
using Rhino.Mocks;

namespace MvvmDataBindingDemoTest
{
    [TestClass]
    public class MainWindowViewModelTest
    {
        Model _model;
        ShipmentCalculator _calc;
        IList<Country> _countries;
        IFocusManager _focusManager;

        [TestInitialize]
        public void Setup()
        {
            _model = MockRepository.GenerateMock<Model>();
            _calc = MockRepository.GenerateMock<ShipmentCalculator>();
            _model.Stub(m => m.ShipmentCalculator).Return(_calc);
            _countries = GetDefaultCountries();

            _model.Stub(m => m.Countries).Return(_countries);
            _focusManager = MockRepository.GenerateMock<IFocusManager>();
        }

        [TestMethod]
        public void ValidationFails_IfNothingIsSpecified()
        {
            var vm = CreateObject();
            vm.ShipCommand.Execute(null);
            Assert.AreEqual("Please specify country", vm.CountryError);
            Assert.AreEqual("Please enter city", vm.CityError);
            Assert.IsFalse(vm.HasSubdivisions);
            Assert.IsNull(vm.SubdivisionError);
            _model.AssertWasNotCalled(m => m.Send(null), opt => opt.IgnoreArguments());
        }

        [TestMethod]
        public void ValidationFails_IfCityNotSpecified()
        {
            var vm = CreateObject();
            vm.Country = USA;
            vm.Subdivision = Connecticut;
            vm.ShipCommand.Execute(null);
            Assert.IsNull(vm.CountryError);
            Assert.AreEqual("Please enter city", vm.CityError);
            Assert.IsTrue(vm.HasSubdivisions);
            Assert.IsNull(vm.SubdivisionError);
            _model.AssertWasNotCalled(m => m.Send(null), opt => opt.IgnoreArguments());
        }

        [TestMethod]
        public void ValidationFails_IfCityAndStateNotSpecified_CountryHasStates()
        {
            var vm = CreateObject();
            vm.Country = Canada;
            vm.ShipCommand.Execute(null);
            Assert.IsNull(vm.CountryError);
            Assert.AreEqual("Please enter city", vm.CityError);
            Assert.IsTrue(vm.HasSubdivisions);
            Assert.AreEqual("Please specify Province", vm.SubdivisionError);
            _model.AssertWasNotCalled(m => m.Send(null), opt => opt.IgnoreArguments());
        }

        [TestMethod]
        public void ValidationFails_IfCityAndStateNotSpecified_CountryDoesNotHaveStates()
        {
            var vm = CreateObject();
            vm.Country = Germany;
            vm.ShipCommand.Execute(null);
            Assert.IsNull(vm.CountryError);
            Assert.AreEqual("Please enter city", vm.CityError);
            Assert.IsFalse(vm.HasSubdivisions);
            Assert.IsNull(vm.SubdivisionError);
            _model.AssertWasNotCalled(m => m.Send(null), opt => opt.IgnoreArguments());
        }

        [TestMethod]
        public void ShipmentOK_IfStateNotSpecified_CountryDoesNotHaveStates()
        {
            _calc.Stub(x => x.GetShipmentCost(null, null)).IgnoreArguments().Return(42m);
            var vm = CreateObject();
            vm.Country = Germany;
            vm.City = "Berlin";
            vm.ShipCommand.Execute(null);
            Assert.IsNull(vm.CountryError);
            Assert.IsNull(vm.CityError);
            Assert.IsFalse(vm.HasSubdivisions);
            Assert.IsNull(vm.SubdivisionError);
            _model.AssertWasCalled(m => m.Send(null), opt => opt.IgnoreArguments());
        }

        [TestMethod]
        public void ModelCalledWithCorrectShipmentInfo_CountryHasStates()
        {
            _calc.Stub(x => x.GetShipmentCost(null, null)).IgnoreArguments().Return(42m);
            Shipment shipment = null;

            _model.Stub(m => m.Send(null)).IgnoreArguments()
                .Do(new Action<Shipment>(arg => { shipment = arg; }));

            var vm = CreateObject();
            vm.Country = USA;
            vm.Subdivision = NewJersey;
            vm.City = "Trenton";
            vm.ShipCommand.Execute(null);

            Assert.IsNotNull(shipment);
            Assert.AreSame(USA, shipment.Country );
            Assert.AreSame(NewJersey, shipment.Subdivision); 
            Assert.AreEqual("Trenton", shipment.City);
            Assert.AreEqual(42m, shipment.Cost);
        }

        [TestMethod]
        public void ModelCalledWithCorrectShipmentInfo_CountryDoesNotHaveStates()
        {
            _calc.Stub(x => x.GetShipmentCost(null, null)).IgnoreArguments().Return(42m);
            Shipment shipment = null;

            _model.Stub(m => m.Send(null)).IgnoreArguments()
                .Do(new Action<Shipment>(arg => { shipment = arg; }));

            var vm = CreateObject();
            vm.Country = Germany;
            vm.City = "Berlin";
            vm.ShipCommand.Execute(null);

            Assert.IsNotNull(shipment);
            Assert.AreSame(Germany, shipment.Country);
            Assert.IsNull(shipment.Subdivision);
            Assert.AreEqual("Berlin", shipment.City);
            Assert.AreEqual(42m, shipment.Cost);
        }

        MainWindowViewModel CreateObject()
        {
            return new MainWindowViewModel( _model ) { FocusManager = _focusManager };
        }

        private static readonly Subdivision NewJersey = new Subdivision { Name="New Jersey", Abbreviation="NJ" };
        private static readonly Subdivision Connecticut = new Subdivision { Name="Connecticut", Abbreviation="CT" };
        private static readonly Subdivision Ontario = new Subdivision { Name="Ontario", Abbreviation="ON" };
        private static readonly Country USA = new Country { Name="USA", SubdivisionName="State", Subdivisions = new List<Subdivision> { Connecticut, NewJersey } };
        private static readonly Country Canada = new Country { Name="Canada", SubdivisionName="Province", Subdivisions = new List<Subdivision> { Ontario } };
        private static readonly Country Germany = new Country { Name="Germany" };

        private static IList<Country> GetDefaultCountries()
        {
            return new List<Country> { USA, Canada, Germany };
        }
    }
}
